***************************************************
*****   Data Management for MSII Estimation   *****
*****   (including the "black" dummy)         ***** 
*****   Last Update: 26 August 2016           *****
***************************************************

clear
set more off
set matsize 800
local data data

set logtype text
log using `data'/data_mgmt_black.txt, replace


************************************
*****  CONSTRUCTING SAMPLE 1   *****
************************************

* Data Used by Card (1995)

* Case 1: keep "KWW"

use `data'/CARD, clear
rename fatheduc feduc
rename motheduc meduc
rename KWW kww
rename IQ iq

keep if kww != .
keep if feduc != .
keep if meduc != .

keep wage educ exper feduc meduc smsa south black kww
order wage educ exper feduc meduc smsa south black kww

outsheet using `data'/s1_c1b.csv, comma replace
save `data'/s1_c1b, replace

* Case 2: keep "IQ"

clear
use `data'/CARD, clear
rename fatheduc feduc
rename motheduc meduc
rename KWW kww
rename IQ iq

keep if iq != .
keep if feduc != .
keep if meduc != .

keep wage educ exper feduc meduc smsa south black iq
order wage educ exper feduc meduc smsa south black iq
outsheet using `data'/s1_c2b.csv, comma replace
save `data'/s1_c2b, replace


************************************
*****  CONSTRUCTING SAMPLE 2   *****
************************************

* Data Used by Blackburn & Neumark (1992)

* Case 1: keep "KWW"

clear
use `data'/WAGE2, clear
rename KWW kww
rename IQ iq
rename urban smsa

keep if feduc != .
keep if meduc != .
sort educ feduc meduc smsa south 
	
foreach x of varlist educ feduc meduc smsa south black {
	
	gen `x'_lag = `x'[_n-1]
	gen `x'_diff = `x'-`x'_lag
	gen flag1_`x' = (`x'_diff == 0)
	gen flag0_`x' = (flag1_`x'[_n+1] == 1)
	
}

gen tie1 = flag1_educ*flag1_feduc*flag1_meduc*flag1_smsa*flag1_south*flag1_black
gen tie0 = flag0_educ*flag0_feduc*flag0_meduc*flag0_smsa*flag0_south*flag0_black
gen obs = _n if tie0+tie1 == 0
replace obs = _n if tie0 == 1 & tie1 == 0
replace obs = obs[_n-1] if obs == .

keep obs tie0 tie1 educ exper feduc meduc smsa south black kww
order obs tie0 tie1 educ exper feduc meduc smsa south black kww 
collapse (mean) educ feduc meduc smsa south black kww, by(obs)
drop obs
outsheet using `data'/s2_bn1b.csv, comma replace
save `data'/s2_bn1b, replace

* Case 2: keep "IQ"

clear
use `data'/WAGE2, clear
rename KWW kww
rename IQ iq
rename urban smsa

keep if feduc != .
keep if meduc != .
sort educ feduc meduc smsa south 
	
foreach x of varlist educ feduc meduc smsa south black {
	
	gen `x'_lag = `x'[_n-1]
	gen `x'_diff = `x'-`x'_lag
	gen flag1_`x' = (`x'_diff == 0)
	gen flag0_`x' = (flag1_`x'[_n+1] == 1)
	
}

gen tie1 = flag1_educ*flag1_feduc*flag1_meduc*flag1_smsa*flag1_south*flag1_black
gen tie0 = flag0_educ*flag0_feduc*flag0_meduc*flag0_smsa*flag0_south*flag0_black
gen obs = _n if tie0+tie1 == 0
replace obs = _n if tie0 == 1 & tie1 == 0
replace obs = obs[_n-1] if obs == .

keep obs tie0 tie1 educ exper feduc meduc smsa south black iq
order obs tie0 tie1 educ exper feduc meduc smsa south black iq 
collapse (mean) educ feduc meduc smsa south black iq, by(obs)
drop obs
outsheet using `data'/s2_bn2b.csv, comma replace
save `data'/s2_bn2b, replace

log close
