cscript
					//----------------------------//
					// get sim result
					//----------------------------//
program get_sim_result, rclass

	mata: get_sim_result()

	forvalues i = 1/3 {
		mat msols_d`i' = return(msols_d`i')
		mat msii_d`i' = return(msii_d`i')
	}
	
	!rm -f msols_result.tex
	forvalues i = 1/3 {
		outtable using msols_result, mat(msols_d`i') append 	///
			caption("MSOLS d3=`i'") nobox f(%9.4f)
	}
	
	!rm -f msii_result.tex
	forvalues i = 1/3 {
		outtable using msii_result, mat(msii_d`i') append 	///
			caption("FM-MSII d3=`i'") nobox f(%9.4f)
	}
end

mata :

void get_sim_result()
{
	real scalar	i
	string scalar	is, n1, n2

	st_rclear()

	for (i=1; i<=3; i++) {
		is = strofreal(i)
		n1 = sprintf("r(msols_d%s)", is)
		n2 = sprintf("r(msii_d%s)", is)
		get_sim_result_d3(i, n1, n2)
		stata("return add")
	}
}


void get_sim_result_d3(			///
	real scalar	myd3,		///
	string scalar	msols_name,	///
	string scalar	msii_name)
{
	real colvector	b_b2, sd_b2, sdhat_b2, r_b2,			///
			b_gamma, sd_gamma, sdhat_gamma, r_gamma,	///
			nn, order, d3, idx, u_nn, u_order
	string colvector	est	
	real matrix	Tb, Tg
	real scalar	i
	string matrix	cnames, rnames
	real matrix	T_msols, T_msii

	// load data
	b_b2 = 1 :- st_data(., "m_b_b2")
	sd_b2 = st_data(., "m_sd_b2")
	sdhat_b2 = st_data(., "sd_b_b2")
	r_b2 = st_data(., "m_r_b2")

	b_gamma = 1 :- st_data(., "m_b_gamma")
	sd_gamma = st_data(., "m_sd_gamma")
	sdhat_gamma = st_data(., "sd_b_gamma")
	r_gamma = st_data(., "m_r_gamma")

	Tb = (b_b2, sd_b2, sdhat_b2, r_b2)
	Tg = (b_gamma, sd_gamma, sdhat_gamma, r_gamma)

	nn = st_data(., "nneighbor")
	order = st_data(., "order")
	d3 = st_data(., "d3")
	est = st_sdata(., "estimator")

	u_nn = uniqrows(nn)
	u_order = uniqrows(order)

	T_msols = J(4, 0, .)
	T_msii = J(4, 0, .)

	// table for MSOLS
	for (i=1; i<=rows(u_nn); i++) {
		idx = (est:== "msols"):*(d3 :== myd3)
		idx = idx:*(nn:==u_nn[i])
		T_msols = T_msols, select(Tb, idx)'
	}

	for (i=1; i<=rows(u_nn); i++) {
		idx = (est:== "msols"):*(d3 :== myd3)
		idx = idx:*(nn:==u_nn[i])
		T_msols = T_msols, select(Tg, idx)'
	}
	
	T_msols = (u_nn', u_nn') \ T_msols

	cnames = J(cols(T_msols), 2, "")
	cnames[1..rows(u_nn), 2] = J(rows(u_nn), 1, "beta2")
	cnames[rows(u_nn)+1..2*rows(u_nn), 2] = J(rows(u_nn), 1, "gamma")

	rnames = J(rows(T_msols), 2, "")
	rnames[., 2] = ("K", "bias(b)", "sd(b)", "mean(sd)", "r")'

	st_matrix(msols_name, T_msols)
	st_matrixrowstripe(msols_name, rnames)
	st_matrixcolstripe(msols_name, cnames)

	// table for MSII
	for (i=1; i<=rows(u_order); i++) {
		idx = (est:== "twostep"):*(d3 :== myd3)
		idx = idx:*(order:==u_order[i])
		T_msii = T_msii, select(Tb, idx)'
	}

	for (i=1; i<=rows(u_order); i++) {
		idx = (est:== "twostep"):*(d3 :== myd3)
		idx = idx:*(order:==u_order[i])
		T_msii = T_msii, select(Tg, idx)'
	}
	
	T_msii = (u_order', u_order') \ T_msii

	cnames = J(cols(T_msii), 2, "")
	cnames[1..rows(u_order), 2] = J(rows(u_order), 1, "beta2")
	cnames[rows(u_order)+1..2*rows(u_order), 2] = 	///
		J(rows(u_order), 1, "gamma")

	rnames = J(rows(T_msii), 2, "")
	rnames[., 2] = ("order", "bias(b)", "sd(b)", "mean(sd)", "r")'

	st_matrix(msii_name, T_msii)
	st_matrixrowstripe(msii_name, rnames)
	st_matrixcolstripe(msii_name, cnames)
}

end
					//----------------------------//
					//  summary simluation results
					//----------------------------//
use result/result_final, clear

bysort estimator d3 nn order: keep if _n <= 1000

bysort estimator d3 nn order: sum *b2 *gamma, sep(3)

collapse (mean) m_b_b2 = b_b2 m_sd_b2 = se_b2 m_r_b2 = r_b2		///
	m_b_gamma = b_gamma m_sd_gamma = se_gamma m_r_gamma = r_gamma 	///
	(sd) sd_b_b2 = b_b2 sd_b_gamma = b_gamma			///
	, by(estimator d3 nn order)
	
get_sim_result






